<?php
/**
 * app/plugins/feedback/controllers/feedback.php
 *
 * This is the home controller for the feedback plugin.
 *
 * @package TP Feedback
 * @version 3.0
 * @author  Joey Kimsey <Joey@thetempusproject.com>
 * @link    https://TheTempusProject.com
 * @license https://opensource.org/licenses/MIT [MIT LICENSE]
 */
namespace TheTempusProject\Controllers;

use TheTempusProject\Houdini\Classes\Views;
use TheTempusProject\Houdini\Classes\Issues;
use TheTempusProject\Classes\Controller;
use TheTempusProject\Classes\Forms;
use TheTempusProject\Bedrock\Functions\Check;
use TheTempusProject\Bedrock\Functions\Input;
use TheTempusProject\Bedrock\Functions\Session;
use TheTempusProject\Hermes\Functions\Redirect;
use TheTempusProject\Models\Feedback as FeedbackModel;

class Feedback extends Controller {
    protected static $feedback;

    public function index() {
        self::$feedback = new FeedbackModel;
        self::$title = 'Feedback - {SITENAME}';
        self::$pageDescription = 'At {SITENAME}, we value our users\' input. You can provide any feedback or suggestions using this form.';
        if ( !Input::exists() ) {
            return Views::view( 'feedback.feedback' );
        }
        if ( !Forms::check( 'feedback' ) ) {
            Issues::add( 'error', [ 'There was an error with your form, please check your submission and try again.' => Check::userErrors() ] );
            return Views::view( 'feedback.feedback' );
        }
        $result = self::$feedback->create( Input::post( 'name' ), Input::post( 'feedbackEmail' ), Input::post( 'entry' ) );
        if ( $result ) {
            Session::flash( 'success', 'Thank you! Your feedback has been received.' );
            Redirect::to( 'home/index' );
        } else {
            Issues::add( 'error', [ 'There was an error with your form, please check your submission and try again.' => Check::userErrors() ] );
        }
    }
}
